package com.gigachatwoot.connectors.chatwoot.handlers;

import an.awesome.pipelinr.Command;
import an.awesome.pipelinr.Voidy;
import com.gigachatwoot.connectors.chatwoot.client.ChatwootClient;
import com.gigachatwoot.connectors.chatwoot.commands.ProcessChatwootMessageCommand;
import com.gigachatwoot.connectors.chatwoot.dto.ChatwootWebhookPayloadDTO;
import com.gigachatwoot.connectors.chatwoot.models.ChatMessage;
import com.gigachatwoot.connectors.llm.services.GigaChatConnectorService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.List;

@Slf4j
@Component
@RequiredArgsConstructor
public class ProcessChatwootMessageHandler implements Command.Handler<ProcessChatwootMessageCommand, Voidy> {

    private final ChatwootClient chatwootClient;
    private final GigaChatConnectorService gigaChatConnectorService;

    @Override
    public Voidy handle(ProcessChatwootMessageCommand command) {
        ChatwootWebhookPayloadDTO payload = command.getPayload();

        // Check if assignee is null and client requests an operator
        if (clientRequestsOperator(payload)) {
            Long agentId = chatwootClient.getRandomAvailableAgentId(payload.getAccountId());
            if (agentId != null) {
                chatwootClient.assignConversationToAgent(payload.getAccountId(), payload.getConversationId(), agentId);
                log.info("Assigned agent with ID {} to conversation {}", agentId, payload.getConversationId());
                return new Voidy();
            } else {
                log.warn("No available agents to assign.");
            }
        }

        // Get message history
        List<ChatMessage> messages = chatwootClient
            .getMessageHistory(payload.getAccountId(), payload.getConversationId());
        if (messages.isEmpty()) {
            log.warn("handle: No messages found in conversation history.");
            return new Voidy();
        }

        // Sort and filter messages
        List<ChatMessage> sortedMessages = chatwootClient.sortAndFilterMessages(messages);
        if (sortedMessages.isEmpty()) {
            log.warn("handle: No valid messages found after filtering.");
            return new Voidy();
        }

        // Display message history to the console (Unnecessary, delete later)
        System.out.println("Message History:\n" + chatwootClient.formatMessagesForConsole(sortedMessages));

        // Send filtered message list to GigaChat to receive a response
        String gigaChatResponse = gigaChatConnectorService.getGigaChatAnswer(sortedMessages);

        // Post a reply to Chatwoot
        chatwootClient.sendMessageToChatwoot(payload.getAccountId(), payload.getConversationId(), gigaChatResponse);

        return new Voidy();
    }

    private boolean clientRequestsOperator(ChatwootWebhookPayloadDTO payload) {
        // Check if the message content contains the keyword "/operator"
        String content = payload.getContent().toLowerCase();
        return content.contains("/operator");
    }
}
