package com.gigachatwoot.connectors.llm.dto;

import com.gigachatwoot.connectors.chatwoot.models.ChatMessage;
import jakarta.validation.constraints.NotNull;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import org.springframework.context.annotation.PropertySource;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Getter
@Setter
@NoArgsConstructor
@Component
@PropertySource("classpath:application.yml")
public class GigaChatRequestDTO {
    private String model;

    private List<GigaChatRequestMessageDTO> messages;

    private Integer n;

    private Boolean stream;

    private Integer update_interval;

    public GigaChatRequestDTO(@NotNull String systemPrompt, @NotNull List<ChatMessage> messageHistory) {
        List<GigaChatRequestMessageDTO> msgList = new ArrayList<>();
        msgList.add(new GigaChatRequestMessageDTO("system", systemPrompt));
        for (ChatMessage message : messageHistory) {
            if (message.getMessageType() == 0) {
                msgList.add(new GigaChatRequestMessageDTO("user", message.getContent()));
            }

            if (message.getMessageType() == 1) {
                msgList.add(new GigaChatRequestMessageDTO("assistant", message.getContent()));
            }
        }

        this.model = "GigaChat";
        this.messages = msgList;
        this.n = 1;
        this.stream = false;
        this.update_interval = 0;
    }

    @Override
    public String toString() {
        return String.format("{model:%s,messages:%s,n:%s,stream:%s,update_interval:%s}", model, messages, n, stream, update_interval);
    }
}
