package com.gigachatwoot.connectors.llm.services;

import com.gigachatwoot.connectors.llm.config.LLMConfig;
import com.gigachatwoot.connectors.llm.dto.GigaChatAuthDTO;
import com.gigachatwoot.connectors.llm.mapper.UnirestMapper;
import com.gigachatwoot.connectors.llm.models.GigaChatAuthRequestModel;
import com.gigachatwoot.connectors.llm.models.GigaChatMessageRequestModel;
import com.gigachatwoot.connectors.llm.strategy.GigaChatConnectorStrategy;
import com.gigachatwoot.connectors.llm.strategy.LLMConnectorStrategy;
//import com.gigachatwoot.connectors.llm.strategy.LLaMaConnectorStrategy;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class LLMConnectorService {
    private final UnirestMapper objectMapper;
    private final GigaChatAuthRequestModel authRequest;
    private final GigaChatAuthDTO authDTO;
    private final GigaChatMessageRequestModel msgRequest;

    private LLMConnectorStrategy llmConnectorStrategy;
    private final LLMConfig llmConfig;

    public LLMConnectorService(UnirestMapper objectMapper, GigaChatAuthRequestModel authRequest, GigaChatAuthDTO authDTO, GigaChatMessageRequestModel msgRequest) {
        this.objectMapper = objectMapper;
        this.authRequest = authRequest;
        this.authDTO = authDTO;
        this.msgRequest = msgRequest;
        llmConfig = new LLMConfig("llama");

        switch(llmConfig.getLlmType()) {
            case "llama":
                // TODO: Implement LLaMa into strategy
                //llmConnectorStrategy = new LLaMaConnectorStrategy(objectMapper);
                break;
            case "gigachat":
                llmConnectorStrategy = new GigaChatConnectorStrategy(objectMapper, authRequest, authDTO, msgRequest);
                break;
            default:
                throw new IllegalArgumentException("LLM type has not been set");
        }
    }

    public String getLLMAnswer() {
        return llmConnectorStrategy.getLLMAnswer();
    }
}
