package com.gigachatwoot.connectors.llm.strategy;

import com.fasterxml.jackson.databind.JsonNode;
import com.gigachatwoot.connectors.chatwoot.models.ChatMessage;
import com.gigachatwoot.connectors.llm.dto.GigaChatAuthDTO;
import com.gigachatwoot.connectors.llm.mapper.UnirestMapper;
import com.gigachatwoot.connectors.llm.models.GigaChatAuthRequestModel;
import com.gigachatwoot.connectors.llm.models.GigaChatMessageRequestModel;
import jakarta.validation.constraints.NotNull;
import kong.unirest.HttpResponse;
import kong.unirest.Unirest;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;

import java.util.ArrayList;
import java.util.List;

import static org.springframework.http.MediaType.APPLICATION_FORM_URLENCODED_VALUE;
import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;

@PropertySource("classpath:application.yml")
public class GigaChatConnectorStrategy extends LLMConnectorStrategy {
    @Value("${gigachat.chat_url}")
    @Getter
    @NotNull(message = "Chat URL is required.")
    private String chatUrl;

    @Value("${gigachat.oauth_url}")
    @Getter
    @NotNull(message = "OAUTH URL is required.")
    private String oauthUrl;

    private final GigaChatAuthRequestModel authRequest;

    private final GigaChatAuthDTO authDTO;

    private final GigaChatMessageRequestModel msgRequest;

    public GigaChatConnectorStrategy(UnirestMapper objectMapper, GigaChatAuthRequestModel authRequest, GigaChatAuthDTO authDTO, GigaChatMessageRequestModel msgRequest) {
        super(objectMapper);
        this.authRequest = authRequest;
        this.authDTO = authDTO;
        this.msgRequest = msgRequest;
    }

    @Override
    public String getLLMAnswer() {
        String accessTokenResponse = getAccessToken();
        List<ChatMessage> messageHistory = new ArrayList<>();
        ChatMessage.Sender sender = new ChatMessage.Sender();
        sender.setType("user");

        ChatMessage testMsg = new ChatMessage();
        testMsg.setId(1L);
        testMsg.setContent("Привет как дела");
        testMsg.setMessageType(0L);
        testMsg.setSender(sender);
        messageHistory.add(testMsg);

        HttpResponse<String> response = Unirest.post(getChatUrl())
            .header("Content-Type", APPLICATION_JSON_VALUE)
            .header("Accept", APPLICATION_JSON_VALUE)
            .header("Authorization", "Bearer " + accessTokenResponse)
            .body(messageToString(msgRequest.getSystemPrompt(), messageHistory))
            .asString();

        return parseLLMContent(
            objectMapper.readValue(response.getBody(), JsonNode.class)
        );
    }

    private String getAccessToken() {
        HttpResponse<String> response = Unirest.post(getOauthUrl())
            .header("Content-Type", APPLICATION_FORM_URLENCODED_VALUE)
            .header("Accept", APPLICATION_JSON_VALUE)
            .header("RqUID", authRequest.getRquid())
            .header("Authorization", authRequest.getAuthorization())
            .field("scope", authDTO.getScope())
            .asString();

        return parseAccessToken(
            objectMapper.readValue(response.getBody(), JsonNode.class)
        );
    }

    public String parseAccessToken(@NotNull com.fasterxml.jackson.databind.JsonNode accessToken) {
        return accessToken.get("access_token").textValue();
    }
}
